;;;; logger.scm - A deride backend to log all activity to files -*- scheme -*-
;;;; Copyright (C) 2001  Neal H Walfield  <neal@cs.uml.edu>
;;;;
;;;; This file is part of Deride
;;;;
;;;; Deride is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2, or (at your option)
;;;; any later version.
;;;; 
;;;; Deride is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with the Deride; see the file COPYING.  If not, write to
;;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(define-module (backend logger))

(use-modules (comm irc))
(use-modules (comm hooks))
(use-modules (forum hooks))

(define-public logger-directory "/home/neal/logs/deride")
(define-public logger-global-log "global")

;; Map of forums to file-ports.  Keys should compare equal using `eqv?'.
(define file-port-list '())

;; Get the file port for the given channel.  If it does not exist,
;; open one.  If there is an error opening it, complain on the error
;; log.  If forum is the error log, then bail.

(define (get-file-port forum)
  (if (string=? forum "")
      (set! forum logger-global-log))
  (let ((forum (string-downcase forum))
	(port (assoc forum file-port-list)))
    (if port
	(cdr port)
	(let ((port (open-file
		     (string-append logger-directory "/" forum) "al")))
	  (set! file-port-list (assoc-set! file-port-list forum port))
	  port))))

;; Write a string out.
(define (client-display forum s)
  (simple-format (get-file-port forum) s))

;; Get the time.  Return a string.
(define (now)
  (strftime "%H:%M" (localtime (current-time))))

;; Communication hook

(add-hook! comm-received-forum-message-hook
	   (lambda (sender forum say msg)
	     (let* ((sep (if say
			     (list "" "")
			     (list "<" ">"))))
	       (client-display forum (string-append (car sep) sender
						    "@" (now)
						    (cadr sep) " " msg
						    "\n")))))

(add-hook! comm-received-personal-message-hook
	   (lambda (sender say msg)
	     (let ((sep (if say "" "*")))
	     (client-display sender (string-append " -> " sep sender
						   "@" (now) sep " "
						   msg "\n")))))

(add-hook! comm-forum-status-message-hook
	   (lambda (forum msg)
	     (client-display forum (string-append "{info@" (now)
						  "} " msg "\n"))))

(add-hook! comm-global-status-message-hook
	   (lambda (msg)
	     (client-display logger-global-log (string-append "{info@" (now)
							      "} " msg "\n"))))

;; Forum hooks

;; Turn LIST, a list of strings into a string with the elements
;; seperated by SEP and the last two elements seperated by FINALSEP.
;; E.g. (list-of-strings->string ("a" "b" "c" "d") ", " " and ")
;;       => "a, b, c and d"
(define (list-of-strings->string list sep finalsep)
  (if (eqv? list '())
      ""
      (string-append
       (if (eqv? (cdr list) '())
	   (car list)
	   (string-append (car list)
			  (if (eqv? (cddr list) '())
			      finalsep
			      sep)
			  (list-of-strings->string (cdr list)
						   sep finalsep))))))

(add-hook! comm-entered-forum
	   (lambda (forum users)
	     (client-display
	      forum
	      (string-append
	       "\nJoined at " (now) " current participants and "
	       "lurkers are: "
	       (list-of-strings->string users ", " " and ")
	       ".\n"))))

(add-hook! forum-user-joined-hook
	   (lambda (user forum reason)
	     (client-display
	      forum (string-append "At " (now) ", " user " joined "
				   forum
				   (if (> (string-length reason) 0)
				       ": " " ")
				   reason "\n"))))

(add-hook! forum-user-parted-hook
	   (lambda (user forum reason)
	     (client-display
	      forum (string-append "At " (now) ", " user " left"
				   (if (> (string-length reason) 0) ": " ".")
				   reason "\n"))))

(add-hook! forum-user-quit-hook
	   (lambda (user forum-list reason)
	     (for-each
	      (lambda (forum)
		(client-display
		 forum
		 (string-append "At " (now) ", " user " quit"
				(if (> (string-length reason) 0) ": " ".")
				reason "\n")))
		forum-list)))

(add-hook! forum-user-schizophrenia-hook
	   (lambda (old new forum-list reason) 
	     (for-each
	      (lambda (forum)
		(client-display
		 forum
		 (string-append "At " (now) ", " old
				" has changed his nick to " new
				(if (> (string-length reason) 0) ": " ".")
				reason "\n")))
		forum-list)))
